from pytest import mark

from dialogue import Dialogue


def uppercaser(input_fn=input):
    """ This REPL outputs the user input uppercased."""
    while True:
        try:
            text = input_fn('> ')
        except EOFError:  # no more inputs
            break
        if text == 'q':  # quit
            break
        print(text.upper())


def test_uppercaser(capsys):
    dlg = Dialogue('> Xyz\nXYZ\n')
    uppercaser(dlg.fake_input)
    captured = capsys.readouterr()
    assert dlg.session == captured.out


@mark.parametrize("session", [
    """
    > q
    """,
    """
    > abc
    ABC
    """,
    """
    > first
    FIRST
    > second
    SECOND
    > third
    THIRD
    """,
])
def test_uppercaser_multiple(capsys, session):
    dlg = Dialogue(session)
    uppercaser(dlg.fake_input)
    captured = capsys.readouterr()
    assert dlg.session == captured.out


def reverser(input_fn=input):
    """ This REPL outputs the user input reversed.

        The prompt includes an incrementing index.
    """
    index = 1
    while True:
        try:
            text = input_fn(f'{index}: ')
        except EOFError:  # no more inputs
            break
        if text == 'exit':  # exit REPL
            break
        print(''.join(reversed(text)))
        index += 1


@mark.parametrize("session", [
    """
    1: exit
    """,
    """
    1: abc
    cba
    """,
    """
    1: 123
    321
    2: anna
    anna
    3: NaN
    NaN
    """,
])
def test_reverser(monkeypatch, capsys, session):
    dlg = Dialogue(session)
    reverser(dlg.fake_input)
    captured = capsys.readouterr()
    assert dlg.session == captured.out



import textwrap


class Dialogue():
    """ ``Dialogue`` allows testing of text-based interactions or REPLs.

        A ``Dialogue`` instance is created with a session argument: a
        multi-line string with prompts, user inputs and expected outputs.
        Each time the ``fake_input`` method is called, it emulates the
        Python 3 ``input`` built-in function by printing a prompt, then
        a user input, and returning that input. Therefore, ``fake_input``
        can be used to mock ``input``, providing pre-recorded "user"
        inputs to the REPL under test.
    """

    def __init__(self, session):
        # dedent session given as indented string (see tests)
        self.session = textwrap.dedent(session.lstrip('\n'))
        self.input_line_gen = iter(self)
        self.prompt = ''

    def __iter__(self):
        for line in self.session.splitlines():
            if line.startswith(self.prompt):
                yield line[len(self.prompt):]

    def fake_input(self, prompt):
        """Use this method to mock the ``input`` built-in."""
        self.prompt = prompt
        try:
            line = next(self.input_line_gen)
        except StopIteration:
            raise EOFError()
        print(prompt + line)
        return line
